@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
-import logging
 from redis import Redis
 from typing import Callable, Dict, Union, List
 from celery import Celery, Task, group
 from celery.result import AsyncResult
+from celery.utils.log import get_task_logger
 from .common.exceptions import BizException
 from .common.utils import obj_to_ref, singleton
 from .common.keys import get_result_key
 from .common.model import Callback
 
-LOGGER = logging.getLogger(__name__)
+LOGGER = get_task_logger(__name__)
 
 
 # @singleton
@@ -34,75 +34,86 @@ class CeleryClient(object):
         self.rds_client = Redis.from_url(url=client_redis_uri, decode_responses=True)
         self.task_map = dict()
 
-    # 异步结果处理函数绑定
-    def _bind_callback(self, task_name: str, async_result: AsyncResult, callback: Callback):
-        rs_key = get_result_key(app_code=self.app_code, task_name=task_name, task_id=async_result.id)
-        self.rds_client.set(rs_key, callback.json(), ex=self.default_expire_seconds)
-
-    def apply_async(self, task_name: str, task_params: Dict = None, callback: Callback = None, **kwargs):
+    def apply_group(self, task_name: str, task_params_list: List[Dict] = None, callback: Callback = None, timeout: float = None, **kwargs):
         """
-        异步请求
-        :param task_name: 任务名
-        :param task_params: 任务参数
-        :param callback: 回调对象
-        :param kwargs: 扩展参数
+        服务组调用
+        :param task_name:
+        :param task_params_list:
+        :param callback:
+        :param kwargs:
         :return:
         """
-        LOGGER.info(f'[服务]异步调用 task={task_name}')
         try:
-            _headers = dict()
-            if callback:
-                _headers.update(callback.dict())
-            _headers['app_code'] = _headers.get('app_code', None) or self.app_code
+            # 同步/异步
+            _headers = {'app_code': self.app_code}
+            if callback and callback.app_code:
+                _headers['app_code'] = callback.app_code or self.app_code
 
-            # 异步调用
-            async_result = self._get_task(task_name).apply_async(kwargs=task_params, queue=f'task.{task_name}', headers=_headers)
-            # 绑定回调处理函数
-            self._bind_callback(task_name, async_result, callback)
-            return async_result
+            # 调用
+            task_list = []
+            for task_params in task_params_list:
+                task_list.append(self.get_task_by_name(task_name).s(**task_params))
+            task_group = group(task_list)
+            async_result = task_group.apply_async(queue=f'task.{task_name}', headers=_headers)
+
+            # 结果
+            if callback:
+                # 【异步】绑定回调处理函数
+                LOGGER.info(f'[服务组]异步调用 task={task_name}')
+                self._bind_callback(task_name, async_result, callback)
+                return async_result
+            else:
+                # 【同步】等待结果
+                LOGGER.info(f'[服务组]同步调用 task={task_name}')
+                _timeout = timeout or 60
+                if not timeout and kwargs and kwargs.get('timeout', None):
+                    _timeout = float(kwargs.get('timeout'))
+                async_result.successful()
+                rs = async_result.get(timeout=_timeout)
+                async_result.forget()
+                return rs
         except Exception as e:
-            LOGGER.error(f'处理异常: task_name={task_name}, kwargs={task_params}, callback={callback}, e={e}')
+            LOGGER.error(f'处理异常: task_name={task_name}, kwargs={len(task_params_list)}, e={e}')
             raise e
 
-    def apply_sync(self, task_name: str, task_params: Dict = None, **kwargs):
+    def apply(self, task_name: str, task_params: Dict = None, callback: Callback = None, timeout: float = None, **kwargs):
         """
-        同步请求
-        :param task_name: 任务名
-        :param task_params: 任务参数
-        :param kwargs: 扩展参数
+        单任务请求调用
+        :param task_name:
+        :param task_params:
+        :param callback:
+        :param timeout:
+        :param kwargs:
         :return:
         """
-        LOGGER.info(f'[服务]同步调用 task={task_name}')
         try:
+            # 同步/异步
             _headers = {'app_code': self.app_code}
-            # 异步调用
-            async_result = self._get_task(task_name).apply_async(kwargs=task_params, queue=f'task.{task_name}', headers=_headers)
-            # 等待结果
-            _timeout = 60
-            if kwargs and kwargs.get('timeout', None):
-                _timeout = float(kwargs.get('timeout'))
-            rs = async_result.get(timeout=_timeout)
-            async_result.forget()
-            return rs
+            if callback and callback.app_code:
+                _headers['app_code'] = callback.app_code or self.app_code
+            # 调用
+            async_result = self.get_task_by_name(task_name).apply_async(kwargs=task_params, queue=f'task.{task_name}', headers=_headers)
+            # 结果
+            if callback:
+                # 【异步】绑定回调处理函数
+                LOGGER.info(f'[服务]异步调用 task={task_name}')
+                self._bind_callback(task_name, async_result, callback)
+                return async_result
+            else:
+                # 【同步】等待结果
+                LOGGER.info(f'[服务]同步调用 task={task_name}')
+                _timeout = timeout or 60
+                if not timeout and kwargs and kwargs.get('timeout', None):
+                    _timeout = float(kwargs.get('timeout'))
+                rs = async_result.get(timeout=_timeout)
+                async_result.forget()
+                return rs
         except Exception as e:
             LOGGER.error(f'处理异常: task_name={task_name}, kwargs={task_params}, e={e}')
             raise e
 
-    # 兼容方式请求调用
-    def apply(self, task_name: str, task_params: Dict = None, callback: Callback = None, **kwargs):
-        """
-        兼容方式请求调用
-        有callback怎为异步
-        """
-        if callback:
-            # 异步
-            return self.apply_async(task_name=task_name, task_params=task_params, callback=callback)
-        else:
-            # 同步
-            return self.apply_sync(task_name=task_name, task_params=task_params, **kwargs)
-
     # 缓存Celery任务对象
-    def _get_task(self, task_name: str):
+    def get_task_by_name(self, task_name: str):
         task = self.task_map.get(task_name)
         if not task:
             task = Task()
@@ -112,6 +123,11 @@ class CeleryClient(object):
 
         return task
 
+    # 异步结果处理函数绑定
+    def _bind_callback(self, task_name: str, async_result: AsyncResult, callback: Callback):
+        rs_key = get_result_key(app_code=self.app_code, task_name=task_name, task_id=async_result.id)
+        self.rds_client.set(rs_key, callback.json(), ex=self.default_expire_seconds)
+
     # 构建回调对象
     def build_callback(self, callback_func: Union[str, Callable] = None, callback_data: Union[str, Dict, List] = None, app_code: str = None, tenant_code: str = None):
         callback = Callback()
