@@ -9,4 +9,4 @@ class BaseService(object):
     """
 
     def __init__(self, celery_client: CeleryClient = None):
-        self.celery_client = celery_client or CeleryClientHolder.default_instance()
+        self.celery_client = celery_client or CeleryClientHolder.get_default_instance()
